import sowerest


class BaseInterface(object):
    def __init__(self, auth, host):
        """
        Construct the SoWeMail API object.
        Note that the underlying client is being set up during initialization,
        therefore changing attributes in runtime will not affect HTTP client
        behaviour.

        :param auth: the authorization header
        :type auth: string
        :param host: base URL for API calls
        :type host: string
        """
        from . import __sowemail_api_version__
        self.auth = auth
        self.host = host
        self.version = __sowemail_api_version__
        self.useragent = 'sowemail/{};python'.format(self.version)

        self.client = sowerest.Client(
            host=self.host,
            request_headers=self._default_headers,
            version=3)

    @property
    def _default_headers(self):
        """Set the default header for a SoWeMail API call"""
        headers = {
            "Authorization": self.auth,
            "User-Agent": self.useragent,
            "Accept": 'application/json'
        }

        return headers

    def reset_request_headers(self):
        self.client.request_headers = self._default_headers

    def send(self, message):
        """Make a SoWeMail API request with the request body generated by the Mail object

        :param message: The SoWeMail API request body generated by the Mail object
        :type message: Mail
        """
        if not isinstance(message, dict):
            message = message.get()

        return self.client.mail.send.post(request_body=message)



"""
This library allows you to quickly and easily use the SoWeMail Rest API via Python.

For more information on this library, see the README on GitHub.
    https://github.com/sowemail/sowemail-python
For more information on the SoWeMail API, see the v1 docs:
    https://sowemail.com/docs/API_Reference/v1.html

Available subpackages
---------------------
helpers
    Modules to help with common tasks.
"""

import os
from .helpers.mail import *  # noqa
from .sowemail import SoWeMailAPIClient  # noqa

dir_path = os.path.dirname(os.path.realpath(__file__))
if os.path.isfile(os.path.join(dir_path, 'VERSION.txt')):
    with open(os.path.join(dir_path, 'VERSION.txt')) as version_file:
        __version__ = version_file.read().strip()

__sowemail_api_version__ = 1



import io
import os
from distutils.file_util import copy_file
from setuptools import setup, find_packages


base_gh_url = 'https://github.com/sowemail/'
dir_path = os.path.abspath(os.path.dirname(__file__))
readme_path = os.path.join(dir_path, 'README.rst')
version_path = os.path.join(dir_path, 'VERSION.txt')

with io.open(readme_path, encoding='utf-8') as readme_file:
    readme = readme_file.read()
with io.open(version_path, encoding='utf-8') as version_file:
    version = version_file.read().strip()

copy_file(version_path,
          os.path.join(dir_path, 'sowemail', 'VERSION.txt'),
          verbose=False)


def parse_requirements(filename):
    """load requirements from a pip requirements file"""
    lineiter = (line.strip() for line in open(filename))
    return [line for line in lineiter if line and
            (not line.startswith("#") and not line.startswith('-'))]


setup(
    name='sowemail',
    version=version,
    author='Fourat ZOUARI',
    author_email='opensource@sowemail.com',
    url='{}sowemail-python'.format(base_gh_url),
    download_url='{}sowemail-python/tarball/{}'.format(base_gh_url, version),
    packages=['sowemail'],
    include_package_data=True,
    install_requires=parse_requirements('install-requirements.pip'),
    tests_require=parse_requirements('test-requirements.pip'),
    license='MIT',
    description='SoWeMail library for Python 3',
    long_description_content_type='text/x-rst',
    long_description=readme,
    keywords=[
        'API',
        'SOWEMAIL'],
    python_requires='>=3.6.*',
    classifiers=[
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ]
)



"""
This library allows you to quickly and easily use the SoWeMail Rest API via Python.

For more information on this library, see the README on GitHub.
    https://github.com/sowemail/sowemail-python
For more information on the SoWeMail API, see the v1 docs:
    https://sowemail.com/docs/API_Reference/v1.html

This file provides the SoWeMail API Client.
"""

import os

from .base_interface import BaseInterface


class SoWeMailAPIClient(BaseInterface):
    """The SoWeMail API Client.

    Use this object to interact with the API. For example:
        mail_client = sowemail.SoWeMailAPIClient(os.environ.get('SOWEMAIL_API_KEY'))
        ...
        mail = Mail(from_email, subject, to_email, content)
        response = mail_client.send(mail)

    For examples and detailed use instructions, see
        https://github.com/sowemail/sowemail-python
    """

    def __init__(
            self,
            api_key=None,
            host='https://api.sowemail.com'):
        """
        Construct the SoWeMail API object.
        Note that the underlying client is being set up during initialization,
        therefore changing attributes in runtime will not affect HTTP client
        behaviour.

        :param api_key: SoWeMail API key to use. If not provided, value
                        will be read from environment variable "SOWEMAIL_API_KEY"
        :type api_key: string
        :param host: base URL for API calls
        :type host: string
        """
        self.api_key = api_key or os.environ.get('SOWEMAIL_API_KEY')
        auth = 'Bearer {}'.format(self.api_key)

        super(SoWeMailAPIClient, self).__init__(auth, host)
